Build Instructions For The Steam Audio FMOD Studio Plugin
========================================================================================================================

Copyright (c) Valve Corporation. All rights reserved.


Overview
------------------------------------------------------------------------------------------------------------------------

Building the Steam Audio plugin for FMOD Studio involves the following steps:

1.  Building the audio engine plugin (`phonon_fmod.dll` on Windows, for example). This step involves compiling
    C++ code, and must be done separately on each supported platform (currently Windows).

2.  _(optional)_ Generating the Unity package file (`SteamAudio_FMODStudio.unitypackage`). This step involves running Unity
    from the command line.

3.  _(optional)_ Generating the `steamaudio_fmod.zip` file for distribution.

### Directory Structure

Below is an overview of the directories contained within the source repository for the Steam Audio FMOD Studio plugin:

```
DIRECTORY                           CONTENTS 

bin/                                Compiled Steam Audio plugin binaries generated by the build scripts. 
    windows-vs2015-x64-release/     Compiled binaries for 64-bit Windows, Release configuration. 
    windows-vs2015-x86-debug/       Compiled binaries for 32-bit Windows, Debug configuration. 
    ...                             Compiled binaries for other platforms, CPU architectures, and configurations. 
build/                              Build scripts. 
    windows-vs2015-x86/             Intermediate build files and Visual Studio solution for 32-bit Windows builds. 
    ...                             Intermediate build files and project/solution/make files for other platforms, CPU architectures, and configurations. 
dist/                               Packaging scripts. 
    products/                       Packaged zip files generated for distribution. 
doc/                                Documentation. 
include/                            Header files for libraries that the Steam Audio FMOD Studio plugin depends on. 
lib/                                Steam Audio API libraries that are needed for the Steam Audio FMOD Studio plugin to run. Created by running setup.py (see below).
src/                                Source code. 
```

Building the Plugin
------------------------------------------------------------------------------------------------------------------------

### Preparing the Repository
After cloning the Steam Audio Git repository, the first step is to download the Steam Audio API libraries and
place them in the correct locations in the Git repository. To do this, run the following from a command
prompt in the `fmod` directory of the repository:

```
python setup.py
```

This will download the appropriate release of the Steam Audio API from GitHub, extract it, and place the files
in the appropriate locations.

### Common Prerequisites
To build the plugin on any platform, you will need to install the following software:

- Python 2.x (<http://www.python.org>)
- CMake 3.8 or later (<https://cmake.org/>)

Both Python and CMake must be added to your search path.

In addition, for each platform you want build the plugin for, you will need to install the appropriate
prerequisites as discussed below.

### Windows Build

#### Windows Prerequisites
Building the plugin for Windows requires the following software:

- Visual Studio 2015 (<https://www.visualstudio.com/vs/older-downloads/>)

#### Build Instructions
To build the plugin on Windows, run the following from a command prompt in the `fmod` directory of the repository:

```
cd build
python build.py
```

This will generate a Visual Studio 2015 solution (`build/windows-vs2015-x64/Phonon.sln` on 64-bit Windows), and build
it in the Release configuration. The following command line parameters can be passed to `build.py` to customize its
behavior:

- `-a architecture` where _architecture_ is either `x86` or `x64`. This controls whether the plugin is built as a 32-bit
  or 64-bit binary. The 32-bit and 64-bit plugins are built using separate solution files.

- `-c configuration` where _configuration_ is either `debug`, `development`, or `release`. This controls which
  configuration is built from the command line. The Visual Studio solution can be used to build any of the
  configurations from within the IDE. The Development configuration is similar to the Release configuration, but is
  optimized for linker performance.

Use `python build.py --help` for a complete list of command line parameters.

The built plugin will be placed in the `bin/windows-vs2015-(architecture)-(configuration)` directory.


Building the Unity Package
------------------------------------------------------------------------------------------------------------------------

Building the Unity package is only required if you plan to use Steam Audio in conjunction with both FMOD Studio and
Unity. If you are using a different game engine, you can skip this step.

Building the Unity package is currently only supported on Windows. To build it, run the following from a command prompt
in the `unity` directory of the repository:

```
cd build
python build.py -i unity
```

Along with building the Unity package for the Unity native audio plugin, this will also generate `SteamAudio_FMODStudio.unitypackage`,
which contains additional scripts and libraries that must be imported into a Unity project when using Steam Audio,
FMOD Studio, and Unity together.

Refer to the `README.md` file in the `unity` directory of the repository for more details.

The Unity package will be placed in the `bin` directory.


Packaging the Plugin
------------------------------------------------------------------------------------------------------------------------

As an optional step, you can package the plugin, including documentation, into a zip file. To do this, run the following
from a command prompt in the `fmod` directory of the repository:

```
cd dist
python deploy.py -c (configuration)
```

Here, _configuration_ is either `debug`, `development`, or `release`. This will compress the plugin files and the
documentation into a zip file, which will be placed in the `dist/products` directory.