Build Instructions For The Steam Audio Unity Plugin
========================================================================================================================

Copyright (c) Valve Corporation. All rights reserved.


Overview
------------------------------------------------------------------------------------------------------------------------

Building the Steam Audio plugin for Unity involves the following steps:

1.  Building the native audio plugin (`audioplugin_phonon.dll` on Windows, for example). This step involves compiling
    C++ code, and must be done separately on each supported platform (currently Windows, Linux, macOS, and Android).

2.  Generating the Unity package file (`SteamAudio.unitypackage`). This step involves running Unity from the command
    line.

3.  _(optional)_ Generating the `steamaudio_unity.zip` file for distribution.

### Directory Structure

Below is an overview of the directories contained within the source repository for the Steam Audio Unity plugin:

```
DIRECTORY                           CONTENTS 

bin/                                Compiled Steam Audio plugin binaries and Unity packages generated by the build scripts. 
    linux-x64-release/              Compiled binaries for 64-bit Linux, Release configuration. 
    windows-vs2015-x86-debug/       Compiled binaries for 32-bit Windows, Debug configuration. 
    ...                             Compiled binaries for other platforms, CPU architectures, and configurations. 
build/                              Build scripts. 
    osx/                            Intermediate build files and Xcode project for macOS builds. 
    windows-vs2015-x86/             Intermediate build files and Visual Studio solution for 32-bit Windows builds. 
    ...                             Intermediate build files and project/solution/make files for other platforms, CPU architectures, and configurations. 
dist/                               Packaging scripts. 
    products/                       Packaged zip files generated for distribution. 
doc/                                Documentation. 
include/                            Header files for libraries that the Steam Audio Unity plugin depends on. 
src/                                Source code. 
    native/                         C++ source code for the native plugin for the audio engine. 
    project/                        C# source code and Unity project for the managed plugin for the game engine and editor. 
```

Building the Native Plugin
------------------------------------------------------------------------------------------------------------------------

### Preparing the Repository
After cloning the Steam Audio Git repository, the first step is to download the Steam Audio API libraries and
place them in the correct locations in the Git repository. To do this, run the following from a command
prompt in the `unity` directory of the repository:

```
python setup.py
```

This will download the appropriate release of the Steam Audio API from GitHub, extract it, and place the files
in the appropriate locations.

### Common Prerequisites
To build the native plugin on any platform, you will need to install the following software:

- Python 2.x (<http://www.python.org>)
- CMake 3.8 or later (<https://cmake.org/>)

Both Python and CMake must be added to your search path.

In addition, for each platform you want build the native plugin for, you will need to install the appropriate
prerequisites as discussed below.

### Windows Build

#### Windows Prerequisites
Building the native plugin for Windows requires the following software:

- Visual Studio 2015 (<https://www.visualstudio.com/vs/older-downloads/>)

#### Build Instructions
To build the plugin on Windows, run the following from a command prompt in the `unity` directory of the repository:

```
cd build
python build.py
```

This will generate a Visual Studio 2015 solution (`build/windows-vs2015-x64/Phonon.sln` on 64-bit Windows), and build
it in the Release configuration. The following command line parameters can be passed to `build.py` to customize its
behavior:

- `-a architecture` where _architecture_ is either `x86` or `x64`. This controls whether the plugin is built as a 32-bit
  or 64-bit binary. The 32-bit and 64-bit plugins are built using separate solution files.

- `-c configuration` where _configuration_ is either `debug`, `development`, or `release`. This controls which
  configuration is built from the command line. The Visual Studio solution can be used to build any of the
  configurations from within the IDE. The Development configuration is similar to the Release configuration, but is
  optimized for linker performance.

Use `python build.py --help` for a complete list of command line parameters.

The built plugin will be placed in the `bin/windows-vs2015-(architecture)-(configuration)` directory.

### Linux Build

#### Linux Prerequisites
Building the native plugin for Linux requires the following software:

- `g++` 4.8 or later
- `g++-multilib` 4.8 or later if building the 32-bit plugin on a 64-bit system

#### Build Instructions
To build the plugin on Linux, run the following from a command prompt in the `unity` directory of the repository:

```
cd build
python build.py
```

This will generate a Makefile (`build/linux-x64-release/Makefile` on 64-bit Linux), and build it. The following command
line parameters can be passed to `build.py` to customize its behavior:

- `-a architecture` where _architecture_ is either `x86` or `x64`. This controls whether the plugin is built as a 32-bit
  or 64-bit binary. The 32-bit and 64-bit plugins are built using separate makefiles.

- `-c configuration` where _configuration_ is either `debug` or `release`. This controls which configuration is built 
  from the command line. Each configuration is built using a separate makefile.

Use `python build.py --help` for a complete list of command line parameters.

The built plugin will be placed in the `bin/linux-(architecture)-(configuration)` directory.

### macOS Build

#### macOS Prerequisites
Building the native plugin for macOS requires the following software:

- Xcode 7.0 or later (available from the Mac App Store)

#### Build Instructions
To build the plugin on macOS, run the following from a command prompt in the `unity` directory of the repository:

```
cd build
python build.py
```

This will generate an Xcode project (`build/osx/Phonon.xcodeproj`), and build it in the Release configuration. The 
following command line parameters can be passed to `build.py` to customize its behavior:

- `-c configuration` where _configuration_ is either `debug` or `release`. This controls which
  configuration is built from the command line. The Xcode project can be used to build any of the
  configurations from within the IDE.

Use `python build.py --help` for a complete list of command line parameters.

The built plugin will be placed in the `bin/osx-(configuration)` directory. All plugins are built as universal binaries
containing both 32-bit and 64-bit versions.

### Android Build
Building the native plugin for Android is currently only supported on Windows.

#### Android Prerequisites
Building the native plugin for Android requires the following software:

- Android SDK for platform 25 (Android 7.1 Nougat) (<https://developer.android.com/studio/index.html>) **The Android SDK
  must be installed to `C:\Android\sdk`**
- Android NDK (download using the Android SDK Manager)

#### NDK Toolchain Setup
Before you can build the native plugin, you must create an Android NDK standalone toolchain. To do this, run the
following from the root directory of the Android SDK:

```
cd ndk-bundle/build/tools
python make_standalone_toolchain.py --arch arm --api 16 --stl libc++ --install-dir ../../../ndk-toolchain-armv7-clang
```

This will set up a toolchain that uses Clang as its compiler and generates 32-bit binaries for ARMv7 CPUs.

#### Build Instructions
To build the plugin, run the following from a command prompt in the `unity` directory of the repository:

```
cd build
python build.py -p android
```

This will generate a Makefile (`build/android-armv7-release/Makefile`), and build it. The following command line
parameters can be passed to `build.py` to customize its behavior:

- `-c configuration` where _configuration_ is either `debug` or `release`. This controls which configuration is built 
  from the command line. Each configuration is built using a separate makefile.

Use `python build.py --help` for a complete list of command line parameters.

The built plugin will be placed in the `bin/android-armv7-(configuration)` directory.


Building the Unity Package
------------------------------------------------------------------------------------------------------------------------

Building the Unity package is currently only supported on Windows. To build it, run the following from a command prompt
in the `unity` directory of the repository:

```
cd build
python build.py -i unity
```

This will copy the latest 64-bit release build of the native plugin for Windows into the Unity project's 
`Assets/Plugins` folder and generate the Unity package. The following command line parameters can be passed to 
`build.py` to customize its behavior:

- `-c configuration` where _configuration_ is either `debug`, `development`, or `release`. This controls which 
  configuration is copied into the Unity project.

- `--ci`. This copies the latest builds of the native plugin for all platforms into the Unity project, before generating
  the Unity package.

If you have already built the native plugin for all platforms and placed the resulting binaries under the `bin`
directory, then `python build.py -i unity --ci` will generate a Unity package containing plugins for all platforms in
a single command.

The Unity package will be placed in the `bin` directory.


Packaging the Plugin
------------------------------------------------------------------------------------------------------------------------

As an optional step, you can package the plugin, including documentation, into a zip file. To do this, run the following
from a command prompt in the `unity` directory of the repository:

```
cd dist
python deploy.py -c (configuration)
```

Here, _configuration_ is either `debug`, `development`, or `release`. This will compress the Unity package and the
documentation into a zip file, which will be placed in the `dist/products` directory.